//This program is licensed under the GPLv3; see license.txt for more info.
//Copyright 2007 Sprite_tm

#include "io.h"
#include <avr/io.h>
#include <util/delay.h>

//Pins
#define LEDB	PB7
#define LEDR	PB6
#define LEDG	PB5
#define BTN1	PB0
#define BTN2	PB1
#define RPM	PD3

//This char is used to blank out certain hands, e.g. in the setup.
volatile char blink;

void initio(void) {
    //Init gpio
    PORTB=(1<<BTN1)|(1<<BTN2);
    DDRB=0;
    PORTD=(1<<RPM);
    DDRD=0;
    //Initialize timer 1. This'll give off an int 60 times per turn of the
    //fan, per ocr1a.
    TCCR1A=0;
    TCCR1B=0xA; //divider=8, reset on ocr1a
    OCR1A=12000/60; //start @ 6000 rpm, will be adjusted later.
    //Initialize int1, for rpm-detection
    MCUCR=0x0C; //int on falling edge
    GIMSK=0x80; //enable int1
    //Initialize timer 0; this is used for the rtc.
    TCCR0A=2; //max = ocr0a
    TCCR0B=3; // div by 64 prescaling
    OCR0A=250; // ocr0a ints 500 times a second now
    //init timer ints
    TIMSK=0x41; //enable t1 ocra int, t0 ocra int
}

//Enable a led
void setled(char l) {
    DDRB&=~((1<<LEDR)|(1<<LEDG)|(1<<LEDB));
    if ((l==LED_R) && ((blink&4)==0)) DDRB|=(1<<LEDR);
    if ((l==LED_G) && ((blink&2)==0)) DDRB|=(1<<LEDG);
    if ((l==LED_B) && ((blink&1)==0)) DDRB|=(1<<LEDB);
    return;
}

//Get the status of the buttons, including repeat-on-hold. Please call this only
//once every 1/4 second or so. Bit 1 and 2 are the buttons, but 7 gets set when
//auto-repeat is active.
unsigned char getbuttons() {
    static char old=0;
    static char oldctr;
    char p=0;

    //Get current button status
    if ((PINB&(1<<BTN1))==0) p|=1;
    if ((PINB&(1<<BTN2))==0) p|=2;

    //Nothing pressed? Reset & return
    if (p==0) {
	oldctr=0;
	old=0;
	return 0;
    }
    
    if (p!=(old&3)) { //First time?
	old=p;
	oldctr=0; //reset counter
	return p; //and reset something
    } else {
	oldctr++; //increase counter
	//Wait 4 ticks before repeating the press
	//If repeating, or returned by 0x80 to indicate this isn't a once-a-time
	//press.
	if (oldctr<4) return 0; else return p|0x80;
    }

}