//This program is licensed under the GPLv3; see license.txt for more info.
//Copyright 2007 Sprite_tm

#include "run.h"
#include "io.h"
#include <avr/interrupt.h>
#include <avr/io.h>

//These routines actually show the hands of the clock.

volatile char rpmdiv;
volatile char pos_red;
volatile char pos_green;
volatile char pos_blue;
volatile unsigned char fanpos;

//This routine gets called 60 times in a fan rotation
ISR(TIMER1_COMPA_vect) {
    char l;
    static char mux;
    //At the next fan position
    if (fanpos<250 || fanpos==255) fanpos++;
    
    //If we exceeded our time, disable leds and die.
    if (fanpos>60) {
	setled(LED_OFF);
	return;
    }
    
    //See which led we should turn on.
    l=LED_OFF;
    if ((59-fanpos)==pos_blue) l=LED_B;
    if ((59-fanpos)==pos_green) l=LED_G;
    if ((59-fanpos)==pos_red) l=LED_R;
    //Handle overlap
    //Disabled; it flickers too much.
#if 0
    mux++;
    if (mux>2) mux=0;
    if ((59-fanpos)==pos_blue && mux==1) l=LED_B;
    if ((59-fanpos)==pos_green && mux==2) l=LED_G;
#endif
    setled(l);
}

//This int is called every time the fan pulls down the RPM wire, which can be
//once, twice or even four times per rotation. The number of times per rotation
//is set up in the rpmdiv variable.
ISR(INT1_vect) {
    static char times;
    times++;
    //Full rotation yet? If not, return.
    if (times<rpmdiv) return;
    times=0;

    //Adjust the speed of the timer1 interrupts by looking at the current fanpos.
    //It should be exactly 60 now; if it's anything else, we're too fast or too
    //slow.
    if (fanpos>60) {
	//oc1a is too quick
	OCR1A+=5;
	if (fanpos>200) OCR1A<<=1; //way too quick, adjust more
    }
    if (fanpos<60) {
	//oc1a is too slow
	OCR1A-=5;
	if (fanpos<10) OCR1A>>=1; //way too slow, adjust more
    }
    //reset fanpos & timer
    fanpos=255; //'-1'
    TCNT1=0;
}
