//This program is licensed under the GPLv3; see license.txt for more info.
//Copyright 2007 Sprite_tm

#include "io.h"
#include "run.h"
#include "rtc.h"
#include <avr/io.h>
#include <avr/eeprom.h>

//The code here is used to set up the clock.

char state=0; //setup state
//offsets of the various hands wrt twelve o'clock.
unsigned char off_m;
unsigned char off_s;
unsigned char off_h;

//Called on startup: reads hardware configuration values from eeprom. Goes
//to hardware setup mode if they're wrong.
void init_offsets() {
    off_h=eeprom_read_byte(1);
    off_m=eeprom_read_byte(2);
    off_s=eeprom_read_byte(3);
    rpmdiv=eeprom_read_byte(4);
    if (off_h>60 || off_m>60 || off_s>60 || rpmdiv>4) {
	//something wrong; goto hardware setup mode
	state=-4;
	rpmdiv=1; off_h=1; off_m=2; off_s=3; //not the same offsets so no overlap
    }
}

//Called four times a second; performs setup etc tasks
void btn_tick(void) {
    unsigned char b;
    
    b=getbuttons();

    if (state<0) { //hardware setup mode; let's not run the time now
	hour=0;
	min=0;
	sec=0;
    }
    
    if (state==-1) { //hardware setup: second hand
	blink^=(1<<1)|(1<<2); //blink inactive hands
	if ((b&3)==1) { //'+' pressed? Move hand.
	    off_s++;
	    if (off_s>60) off_s=0; //wrap around
	}
	if (b==2) { //'adv' pressed? Save and adjust next hand.
	    eeprom_write_byte(3,off_s);
	    blink=0;
	    state++;
	    return; 
	}
    }
    if (state==-2) { //hardware setup: minute hand
	blink^=(1<<0)|(1<<2);
	if ((b&3)==1) {
	    off_m++;
	    if (off_m>60) off_m=0;
	}
	if (b==2) {
	    state++; 
	    blink=0;
	    eeprom_write_byte(2,off_m);
	}	    
    }
    if (state==-3) { //hardware setup: hour hand
	blink^=(1<<0)|(1<<1);
	if ((b&3)==1) {
	    off_h++;
	    if (off_h>60) off_h=0;
	}
	if (b==2) {
	    state++; 
	    blink=0;
	    eeprom_write_byte(1,off_h);
	}	    
    }
    if (state==-4) { //Hardware setup: rpm-pulses per fan rotation
	blink=3;
	if (b==1) { //next setting
	    //If changing setting, directly modify timer1 setup to save user
	    //from time-taking auto-readjustments.
	    if (rpmdiv==4) { rpmdiv=1; OCR1A/=4; return; }
	    if (rpmdiv==1) { rpmdiv=2; OCR1A*=2; return; }
	    if (rpmdiv==2) { rpmdiv=4; OCR1A*=2; return; }
	}
	if (b==2) { //'adv' pressed: save & continue
	    state++; 
	    blink=0;
	    eeprom_write_byte(4,rpmdiv);
	}
    }
    
    if (state==0) { //Adjust hours
	if ((b&3)==1) { //'+' pressed?
	    hour++; //Move hand
	    if (hour>11) hour=0; //wrap-around
	    blink=0; //Stop blinking
	} else {
	    blink^=4; //Blink hour-hand
	}
	if (b==2) {  //'adv' pressed? Modify next hand
	    state++; 
	    blink=0;
	    return; 
	}
    }
    if (state==1) { //adjust minutes
	if ((b&3)==1) {
	    min+=1;
	    if (min>59) min=0;
	    blink=0;
	} else {
	    blink^=2;
	}
	if (b==2) { 
	    state++; 
	    blink=0;
	    return; 
	}
    }
    
    //Calculate hand positions using hour, minute and second settings.
    pos_red=off_h+(hour*5+(min/12));
    if (pos_red>59) pos_red=pos_red-60;
    pos_green=off_m+min;
    if (pos_green>59) pos_green=pos_green-60;
    pos_blue=off_s+sec;
    if (pos_blue>59) pos_blue=pos_blue-60;
}
